@@ -51,6 +51,7 @@ static INPUT_RETURN_VALUE FcitxUnikeyDoInput(void* arg, FcitxKeySym sym, unsigne
 static boolean FcitxUnikeyInit(void* arg);
 static void FcitxUnikeyReset(void* arg);
 static void FcitxUnikeyResetUI(void* arg);
+static void FcitxUnikeySave(void* arg);
 static INPUT_RETURN_VALUE FcitxUnikeyDoInputPreedit(FcitxUnikey* unikey, FcitxKeySym sym, unsigned int state);
 static void FcitxUnikeyEraseChars(FcitxUnikey *unikey, int num_chars);
 static void  FcitxUnikeyUpdatePreedit(FcitxUnikey *unikey);
@@ -142,6 +143,7 @@ void* FcitxUnikeyCreate(FcitxInstance* instance)
     iface.ResetIM = FcitxUnikeyReset;
     iface.DoInput = FcitxUnikeyDoInput;
     iface.ReloadConfig = ReloadConfigFcitxUnikey;
+    iface.Save = FcitxUnikeySave;
 
     FcitxInstanceRegisterIMv2(
         instance,
@@ -187,6 +189,7 @@ void FcitxUnikeyReset(void* arg)
 
     UnikeyResetBuf();
     unikey->preeditstr->clear();
+    FcitxUnikeyUpdatePreedit(unikey);
 }
 
 void FcitxUnikeyCommit(FcitxUnikey* unikey)
@@ -239,7 +242,9 @@ INPUT_RETURN_VALUE FcitxUnikeyDoInputPreedit(FcitxUnikey* unikey, FcitxKeySym sy
         FcitxUnikeyCommit(unikey);
         return IRV_FLAG_FORWARD_KEY;
     }
-
+    else if (state & FcitxKeyState_Super) {
+        return IRV_TO_PROCESS;
+    }
     else if ((sym >= FcitxKey_Caps_Lock && sym <= FcitxKey_Hyper_R)
             || (!(state & FcitxKeyState_Shift) && (sym == FcitxKey_Shift_L || sym == FcitxKey_Shift_R))  // when press one shift key
         )
@@ -267,7 +272,6 @@ INPUT_RETURN_VALUE FcitxUnikeyDoInputPreedit(FcitxUnikey* unikey, FcitxKeySym sy
             else
             {
                 FcitxUnikeyEraseChars(unikey, UnikeyBackspaces);
-                FcitxUnikeyUpdatePreedit(unikey);
             }
 
             // change tone position after press backspace
@@ -287,8 +291,8 @@ INPUT_RETURN_VALUE FcitxUnikeyDoInputPreedit(FcitxUnikey* unikey, FcitxKeySym sy
                 }
 
                 unikey->auto_commit = false;
-                FcitxUnikeyUpdatePreedit(unikey);
             }
+            FcitxUnikeyUpdatePreedit(unikey);
         }
         return IRV_DISPLAY_MESSAGE;
     } // end capture BackSpace
@@ -493,6 +497,7 @@ static void  FcitxUnikeyUpdatePreedit(FcitxUnikey *unikey)
     }
     FcitxMessagesAddMessageAtLast(clientPreedit, MSG_INPUT, "%s", unikey->preeditstr->c_str());
     FcitxInputStateSetClientCursorPos(input, unikey->preeditstr->size());
+    FcitxUIUpdateInputWindow(unikey->owner);
 }
 
 CONFIG_DESC_DEFINE(GetUnikeyConfigDesc, "fcitx-unikey.desc")
@@ -530,6 +535,10 @@ void ConfigUnikey(FcitxUnikey* unikey)
     UnikeySetInputMethod(unikey->config.im);
     UnikeySetOutputCharset(Unikey_OC[unikey->config.oc]);
     UnikeySetOptions(&unikey->ukopt);
+    char* userFile = NULL;
+    FcitxXDGGetFileUserWithPrefix("unikey", "macro", NULL, &userFile);
+    UnikeyLoadMacroTable(userFile);
+    free(userFile);
 
     UpdateUnikeyUI(unikey);
 }
@@ -566,6 +575,13 @@ void FcitxUnikeyResetUI(void* arg)
     FcitxUISetStatusVisable(instance, "unikey-macro", visible);
 }
 
+void FcitxUnikeySave(void* arg)
+{
+    FcitxUnikey* unikey = (FcitxUnikey*) arg;
+    if (!unikey->preeditstr->empty())
+        FcitxUnikeyCommit(unikey);
+}
+
 void UpdateUnikeyConfig(FcitxUnikey* unikey)
 {
     ConfigUnikey(unikey);
